package App::Controller;

use strict;
use warnings;

use Carp;
use Data::Dumper;
use Digest::MD5;
use XML::Simple qw/:strict/;
#use Net::SMTP;

use App::ListBase;

use vars qw(%c $session $cgi);
*c = \*app::c;
*session = \$app::session;
*cgi = \$app::cgi;

my $sample_image_b64 = <<EOM ;
R0lGODlhFAAWAOMAAP////+Zmf8zM8z//8zMzJmZmWYAADMzMwAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAACH+TlRoaXMgYXJ0IGlzIGluIHRoZSBwdWJsaWMgZG9tYWluLiBLZXZpbiBIdWdoZXMsIGtl
dmluaEBlaXQuY29tLCBTZXB0ZW1iZXIgMTk5NQAh+QQBAAADACwAAAAAFAAWAAAEeHDISau9OOuN
OtoUEozCB4pAIAiGmYlBqrLuhQiqauz1dBQFEW5FQwCBB8kBALgRdy0EExBUMp0GVlTKrA6WV2ir
NaWawGEPd+pFXz3l7pl9rJfbTAKhfiSw53l7ggV+f1ZTeomFhl9xjneAj45tP3yWSDVqmpsmEQA7
Cg==
EOM


sub run
{
  my $class = shift;
  my $this = { web => undef, @_ };
  bless $this, $class;
  # path parts e.g. user/list
  confess unless defined $c{path_info};
  my $path_info = $c{path_info};
  $path_info =~ s|^(\/)+||; # remove leading slashes if any
  my @pp = split /\//, $path_info;
  my $p = $pp[0]||'';  
  my $pmeth = 'c_'.$p;
  if ( $p eq '' ) # root
  {
    $this->root;
  }
  elsif ( $this->can($pmeth) )
  {
    $this->$pmeth([splice(@pp,1)]);
  }
  else
  {
    $this->wrong;
  }
}

sub wrong
{
  my $this = shift;
  $this->{web}->set_error_response( 'Unknown URI path: ' . $c{path_info} );
}

sub root
{
  my $this = shift;
  $c{template} = 'home.tt2';
  #$c{v}->{message} = 'world';
  #$this->{web}->set_wrapped_xhtml_response('root');
  #$c{wraptemplate} = 'xhtml.tt2';
}

# debug

sub dumpcgi
{
  my $s = '<b>cgi</b><br>';
  for ( $cgi->param )
  {
    $s .= "$_: ".$cgi->param($_)."<br>\n";
  }
  $s;
}

sub debug
{
  $c{v}->{debug} ||= '';
  return unless @_;
  unless ($c{v}->{debug})
  {
    $c{v}->{debug} .= dumpcgi() . '<b>messages</b><br>';
  }  
  for (@_)
  {
    $c{v}->{debug} .= $_ . "<br>\n";
  }  
}

sub p
{
  my $param = shift||confess "expected param";
  my $r = $cgi->param($param)||'';
  $r;
}

sub resetpasswordurl
{
  my $this = shift;
  my $login = shift;
  $c{hosturl}.$c{baseurl}.'do_reset_password/'.xe($login).'/'.Digest::MD5::md5_hex(($c{reset_password_secret}||'merl571j0h').$login);
}

sub checkresetpasswordhash
{
  my $this = shift;
  my $login = shift;
  my $hash = shift;
  return (Digest::MD5::md5_hex(($c{reset_password_secret}||'merl571j0h').$login) eq $hash) ? 1 : 0;
}

sub get_random_password
{
   #my @chars = ("A".."Z", "a".."z", 0..9, qw(! @ $ % ^ & *));
   my @chars = ("A".."Z", "a".."z", 0..9);
   srand;
   join("", @chars[ map { rand @chars } (1..8) ]);
}

sub set_xhtml_error_if_not_logged_in
{
  my $this = shift;
  my $cmd = shift ||' ';
  return 0 if $session->param('user')->{is_logged_in};
  $this->{web}->set_xhtml_response(
    'Cannot run '.$cmd.': not logged in'
  );
  return 1;
}

# TODO escape all template data for HTML or SQL insertion attacks

#= URI mapped methods from URL subpath to c_SUBPATH, e.g. /login/ -> c_login

sub c_login
{
  my $this = shift;
  my $pp = shift;

  # check for login from form if not logged in
  my $user = $session->param('user');
  if ( ! $user->{is_logged_in} )
  {
    if ( p('button') eq 'Login' )
    {
      my $login = p('login');
      my $pass = p('password');
      #
      my $rsuser = $c{schema}->resultset('User')->find({ login => $login });
      # only allow admin users to login to the Admin system
      #if ( $rsuser && $rsuser->password eq Digest::MD5::md5_hex($pass) )
      if ( $rsuser && $rsuser->password eq Digest::MD5::md5_hex($pass) && $rsuser->is_admin eq 'yes' )
      {
        $this->{web}->set_session_user_from_rs( $rsuser );
        push @{$c{v}->{message}}, 'You are now logged in';
      }
      else
      {
        $this->{web}->set_session_user_from_rs( undef );
        push @{$c{v}->{message}}, 'Incorrect login/password';
        $c{v}->{show_password_reminder} = 1;
      }  
    }
  }
  
  # back to root if logged in
  if ( $user->{is_logged_in} )
  {
    $this->{web}->redirect('/');
    # $c{v}->{message} will be stashed in session and retrieved after the redirect
  }
  else # show login screen
  {
    #$c{v}->{formaction} = $c{path_info};
    $c{v}->{formaction} = $c{baseurl}.'login'; # TODO uri_for
    $c{template} = 'login.tt2';
  }  
}

sub c_logout
{
  my $this = shift;
  $c{template} = 'login.tt2';
  my $user = $session->param('user');
  $user->{is_logged_in}  = 0;
  $session->param('user', $user);
  #push @{$c{v}->{message}}, 'Logged out';
  #$this->{web}->set_wrapped_xhtml_response('logout');
  #$this->root;
  $this->{web}->redirect('/');
}

sub c_changepassword
{
  my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();

  #$c{v}->{formaction} = $c{baseurl}.'changepassword'; # TODO uri_for
  $c{template} = 'changepassword.tt2';

  if ( p('button') eq 'Change Password' )
  {
    my $oldpass = p('old_password');
    my $newpass = p('new_password');
    my $newpass2 = p('new_password2');
    #
    my $rsuser = $c{schema}->resultset('User')->find({ login => $session->param('user')->{login} });
    if ( $rsuser && $rsuser->password eq Digest::MD5::md5_hex($oldpass) )
    {
      if ( $newpass && $newpass eq $newpass2 )
      {
        $rsuser->update({ password => Digest::MD5::md5_hex($newpass) });
        push @{$c{v}->{message}}, 'You successfully changed your password';
        $this->{web}->redirect('/');
      } 
      elsif ( $newpass )
      {
        push @{$c{v}->{message}}, 'The new passwords did not match';
      }
      else
      {
        push @{$c{v}->{message}}, 'Please enter your new password';
      }  
    } 
    else
    {
      push @{$c{v}->{message}}, 'Incorrect old password';
    }   
  }
}

sub c_do_reset_password
{
  my $this = shift;
  my $pp = shift;
  my $login = $pp->[0];
  my $hash = $pp->[1];
  my $rsuser = $c{schema}->resultset('User')->find({ login => $login });
  if ( $rsuser && $this->checkresetpasswordhash($login, $hash) )
  {
    my $newpass = get_random_password();
    $rsuser->update({ password => Digest::MD5::md5_hex($newpass) });

    my $body = <<EOM ;
<html>Your password has been reset on the Sample App website <a href="http://www.mysampleapp.com/">www.mysampleapp.com</a><br>
<b>Your login:</b> $login<br>
<b>Your new password:</b> $newpass<br>
<p>
<font size=\"-1\">This is an automatic email from the Sample App website <a href=\"http://www.mysampleapp.com/\">www.mysampleapp.com</a></font>
</html>
EOM

     my ($code, $msg) = $this->{web}->send_mail(
      to        => $login,
      from      => $c{'reg_mail_from'},
      from_name => $c{'reg_mail_from_name'},
      subject   => 'Sample App Registration',
      body      => $body,
      );  

    push @{$c{v}->{message}}, 'We have reset your password and emailed you a new one. Please close this browser Window and check your email to log on to the Sample App website';   
  }
  else
  {
    push @{$c{v}->{message}}, 'Sorry, the password reset failed. Please email '.$c{editor_email}.' for further assistance.';
  }  
  #$this->{web}->unset_template;
  $c{view}->{show_menu} = 0;
  $c{template} = 'do_reset_password.tt2';
}

sub c_resetpassword
{
  my $this = shift;
  if ( p('button') eq 'Reset Password' )
  {
    my $login = p('login');
    my $rsuser = $c{schema}->resultset('User')->find({ login => $login });
    if ($rsuser)
    {
      my $resetlink = $this->resetpasswordurl($login);
      my $body = <<EOM ;
<html>You (or possibly someone pretending to be you) has asked to reset your password on the Sample App website <a href="http://www.mysampleapp.com/app/">www.mysampleapp.com/app/</a>
<p>
If this was you, please click the following link to reset your password <a href="$resetlink">reset</a>
or if that doesn't work open a web browser window and type in the following address: $resetlink
<p>
If this was not you, please ignore this email.
<p>
<font size=\"-1\">This is an automatic email from the Sample App website <a href=\"http://www.mysampleapp.com/\">www.mysampleapp.com</a></font>
</html>
EOM

      my ($code, $msg) = $this->{web}->send_mail(
        to        => $login,
        to_name   => $rsuser->name,
        from      => $c{'reg_mail_from'},
        from_name => $c{'reg_mail_from_name'},
        subject   => 'Sample App Website Password Reset',
        body      => $body,
        );  

      if ($code)
      {
        push @{$c{v}->{message}}, 'We have sent an email to you. Please follow the instructions in there to reset your password';
        return $this->root;
      }
      else
      {
        push @{$c{v}->{message}}, 'System error - we were unable to send you a password reset email';
      }
    }
    else
    {
      push @{$c{v}->{message}}, 'Unknown login email address';
    }  
  }

  #$c{v}->{formaction} = $c{baseurl}.'resetpassword'; # TODO uri_for
  $c{template} = 'resetpassword.tt2';
}

sub c_user
{
  my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();
  my $pp = shift;
  my $p = $pp->[0] ||'';
  $c{v}->{record} = 'picture';
  $c{v}->{formaction} = $c{baseurl}.'user/list'; # TODO uri_for
  my $listuser = App::ListBase->new(
    web => $this->{web},
    listdef_name => 'user.ld', 
    baseurl => $c{baseurl},
    #viewurl => '/picture/edit',
    params => {
  });
	$listuser->{listdef}->{view}->{autorun} = 1;
	my $l = $listuser->display_str({});
	$c{v}->{list} = $l; #
  $c{template} = 'record/list.tt2';
}

sub c_download
{
  my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();
  $c{template} = 'download.tt2'
}

 sub c_download_users
{
	my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();
  my $download_name = 'users';
  my $rs = $c{schema}->resultset('User')->search({}, { order_by => 'login' });
	unless ($rs)
	{
		$this->{web}->set_csv_response("No users found", $download_name);
		return;
	}
	my $res = '';
  for my $user ($rs->all)
	{
	  $res .= join(',', map { "".$user->get_column($_) } $this->{web}->user_fields ) . "\015\012";
	}
	$res ||= 'No users';
  $res = "Sample App Users\015\012" . join(',', $this->{web}->user_fields ) . "\015\012" . $res;
  $this->{web}->set_csv_response($res, $download_name);
}

sub c_download_faq_asked
{
  my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();
  my $download_name = 'faqs_asked';
  # select * from faq_asked
  my $rs = $c{schema}->resultset('FaqAsked')->search({}, { order_by => 'id DESC' });
  unless ($rs)
  {
    $this->{web}->set_csv_response("No FAQs have been asked yet", $download_name);
    return;
  }
  my $res = '';
  for ($rs->all)
  {
    my $dt = $_->update_dt;
    $dt =~ s/T/ /;
    $res .= join(',', $_->id, $_->question, $_->login, $_->name, $_->location, $dt) . "\015\012";
  }
  $res ||= 'No FAQs have been asked yet';
  $res = "Sample App Website All FAQs Asked\015\012ID,Question,Login/Email,Name,Location,Date\015\012" . $res;
  $this->{web}->set_csv_response($res, $download_name);
}

sub c_faq
{
  my $this = shift;
  return if $this->set_xhtml_error_if_not_logged_in();
  my $pp = shift;
  my $p = $pp->[0] ||'';

  if ( p('button') eq 'List' )
  {
    $this->{web}->redirect('faq/list');
  }  

  if ( $p eq 'edit' )
  {
    my $id = $pp->[1]||'';
    my $rs = $c{schema}->resultset('Faq')->find($id);
    unless ($rs)
    {
      push @{$c{v}->{message}}, 'Record with ID '.$id.' not found.';
      $this->{web}->redirect('faq/list');
    }
    for (qw/ id question answer name location /)
    {
      $c{v}->{$_} = $rs->get_column($_);
    }  
    if ( p('button') eq 'Update' )
    {
      my $isvalid = 1;
      for ( qw/ question answer name location / )
      {
        $c{v}->{$_} = p($_);
        $isvalid = 0 unless p($_) ne '';
      }
      my $question = p('question');
      my $answer = p('answer');
      # avoid &#146; in Flash
      $question =~ s/\'/\`/g;
      $answer =~ s/\'/\`/g;
      if ( $isvalid )
      {
        $rs->update({
          question    => $question,
          answer      => $answer,
          name        => p('name'),
          location    => p('location'),
          });
        push @{$c{v}->{message}}, 'Updated record';
      }
      else
      {
        push @{$c{v}->{message}}, 'Please enter all fields.';
      }
    }  
    elsif ( p('button') eq 'Delete' )
    {
      my $rs = $c{schema}->resultset('Faq')->find($id);
      if ($rs)
      {
        $rs->delete;
        push @{$c{v}->{message}}, 'Deleted record';
      }
      else
      {
        push @{$c{v}->{message}}, 'Record with ID '.$id.' not found.';
      }
      $this->{web}->redirect('faq/list');
    }  
    $c{v}->{formaction} = $c{baseurl}.'faq/edit/'.$id;
    $c{template} = 'faq_edit.tt2';
  }  
  elsif ( $p eq 'add' )
  {
    #$c{v}->{formaction} = $c{baseurl}.'faq/'.join('/',@$pp);
    #$c{v}->{formaction} = $c{path_info};
    $c{v}->{formaction} = $c{baseurl}.'faq/add';
    $c{template} = 'faq_add.tt2';
    if ( p('button') eq 'Update' )
    {
      my $isvalid = 1;
      for (qw/ question answer name location /) # TODO HTML::Widget + Form::Validator
      {
        $c{v}->{$_} = p($_);
        $isvalid = 0 unless p($_) ne '';
      }
      if ( $isvalid )
      {
        my $rs = $c{schema}->resultset('Faq')->create({
          create_dt   => curtime(),
          question    => p('question'),
          answer      => p('answer'),
          name        => p('name'),
          location    => p('location'),
          });
        my $id = $rs->id;
        push @{$c{v}->{message}}, 'Added record '.$id;
        $this->{web}->redirect('faq/edit/'.$id);
      }
      else
      {
        push @{$c{v}->{message}}, 'Please fill in all fields.';
      }
    }
  }
  else
  {
    $c{v}->{record} = 'faq';
    $c{v}->{formaction} = $c{baseurl}.'faq/list'; # TODO uri_for
    my $list = App::ListBase->new(
      web => $this->{web},
      listdef_name => 'faq.ld', 
      baseurl => $c{baseurl},
      viewurl => '/faq/edit',
      params => {
    });
  	$list->{listdef}->{view}->{autorun} = 1;
  	$c{v}->{list} = $list->display_str({
  	  buttons2 => <<EOM
<a href='$c{baseurl}faq/add' title="Add New [Alt+A]" accessKey="A" class="button">Add New</a>
EOM
  	  });
  	#$l =~ s/\015//g;
  	#$c{v}->{list} = $l; #
    $c{template} = 'record/list.tt2';
  }
}

sub c_picture
{
  my $this = shift;
  #return if $this->set_xhtml_error_if_not_logged_in();
  my $pp = shift;
  my $p = $pp->[0] ||'';

  $c{v}->{record} = 'picture';
  $c{v}->{formaction} = $c{baseurl}.'picture/list'; # TODO uri_for
  my $listpic = App::ListBase->new(
    web => $this->{web},
    listdef_name => 'picture.ld', 
    baseurl => $c{baseurl},
    #viewurl => '/picture/edit',
    params => {
  });

  my $accrejdel =
    (p('button') eq 'Accept Ticked') ? 'accepted'
  : (p('button') eq 'Reject Ticked') ? 'rejected'
  : (p('button') eq 'Delete Ticked') ? 'deleted'
  : '';
  if ( $accrejdel )
  {
    my @ids = $cgi->param($listpic->{id}.'_mass[]');
    my $i;
    if (@ids)
    {
      for (@ids)
      {
        my $rs = $c{schema}->resultset('Picture')->find($_);
        next unless $rs;
        if ( $accrejdel eq 'deleted' )
        {
          # delete image file and image thumbnail file
          # from photo_dir/<MON>/<ID>.jpg and photo_dir/<MON>_thumb/<ID>.jpg (or drawing_dir)
          my $id = $rs->id;
          my $file = $c{picture_dir}||confess "app.conf: need to set picture_dir";
          my $thumbfile = $file . '_thumb/'.$id.'.jpg';
          $file .= '/'.$id.'.jpg';
          unlink ($thumbfile) || warn "c_picture: cannot remove image thumbnail file: $thumbfile: $!";
          unlink ($file) || warn "c_picture: cannot remove image file: $file: $!";

          # delete database record
          $rs->delete;       
        }
        else
        {  
          my $desc = $cgi->param($listpic->{id}.'_description__'.$_);
          $rs->update({ status => $accrejdel, description => $desc });
        }
        $i++;
      }
      push @{$c{v}->{message}}, 'Updated '.$i.' records';

      # recreate httpdocs/photos.xml and drawings.xml used by SlideShowPro Flash plugin
      $this->regenerate_picture_indexes if $i > 0;
    }
    else
    {
      push @{$c{v}->{message}}, 'Please tick some pictures first.';
    }
  }

	$listpic->{listdef}->{view}->{autorun} = 1;
	my $l = $listpic->display_str({});
	$c{v}->{list} = $l; #
  $c{template} = 'picture_list.tt2';
}

sub regenerate_picture_indexes
{
  my $this = shift;
  my %arg = (@_);

  1;
}

#= Flash handlers

=head2 xe - safely encode XML string

Avoids HTML insertion attacks aimed at breaking into the system

=cut

sub xe { App::Web::escape_html(shift); }

sub curtime
{
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  sprintf "%04d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec;
}

=head2 set_xml_error_if_not_logged_in

Convenience method for those commands requiring the user to be logged in
that checks login status and if failed sends a standard XML formatted error message

Return: true = logged in, false = not logged in and error set

=cut

sub set_xml_error_if_not_logged_in
{
  my $this = shift;
  my $cmd = shift ||' ';
  return 0 if $session->param('user')->{is_logged_in};
  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="NOK" reason="not logged in" session_id="'. $session->id . '" />'
  );
  return 1;
}

sub set_xml_error_if_empty_param
{
  my ($this, $cmd, $r, @params) = @_;
  my $bad = '';
  for (@params)
  {
    $bad .= $_ . ' ' unless $r->{$_};
  }
  return 0 unless $bad;
  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="NOK" reason="expected parameters '.$bad.'" session_id="' . $session->id . '" />'
  );    
  return 1;
}

sub set_xml_error_if_missing_param
{
  my ($this, $cmd, $r, @params) = @_;
  my $bad = '';
  for (@params)
  {
    $bad .= $_ . ' ' unless defined $r->{$_};
  }
  return 0 unless $bad;
  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="NOK" reason="expected parameters '.$bad.'" session_id="' . $session->id . '" />'
  );    
  return 1;
}

=head2 c_flash - main Flash handler entry point

unpacks parameters from POSTed XML and uses cmd=HANDLERNAME attribute to branch to handler cf_HANDLERNAME

=cut

sub c_flash
{
  my $this = shift;
  my $p = shift;
  my $part = $p->[0]||'';
  if ( $part eq 'buy_mysticker' )
  {
    $this->cf_buy_mysticker;
    return;
  }  
  my $xml = $cgi->param('POSTDATA')||'';
  my $r = $xml ? XMLin($xml, ForceArray => 1, KeyAttr => {}) : undef;
  my $cmd = $r->{cmd}||'';
  if ( $r->{cmd} )
  { 
    my $pmeth = 'cf_'.$cmd;
    if ( $this->can($pmeth) )
    {
      $this->$pmeth($cmd, $r);
    }
    else
    {
      $this->{web}->set_error_response( 'Flash handler unknown cmd: ' . $cmd );
    }
  }
  else
  {
    $this->{web}->set_error_response( 'Flash handler bad POST data: ' . $xml );
    #open(LOG, ">>/tmp/app_xml.txt");
    #print LOG now(), "\n", join("\n", map { "$_  =>  ".$cgi->param($_) } $cgi->param), "\n";
    #close(LOG);
    return;
  }
}

# subsidiary Flash handlers

#<request cmd="register" user="fred@bloggs.com" password="bloggs" />
#<response cmd="login" result="NOK" reason="invalid user" />
#<response cmd="login" result="OK" />
sub cf_login
{
  my ($this, $cmd, $r) = @_;
  my $login = $r->{'user_email'}||'';
  my $pass = $r->{'user_password'}||'';
  my $rsuser = $login ? $c{schema}->resultset('User')->find({ login => $login }) : undef;
  if ( $rsuser && $rsuser->password eq Digest::MD5::md5_hex($pass) )
  {
    $this->{web}->set_session_user_from_rs( $rsuser );
    $this->{web}->set_xml_response(
      '<response cmd="'.$cmd.'" result="OK" session_id="' . $session->id . '" '
    . 'user_email="'. xe($rsuser->login) . '" '
    . 'location="'  . xe($rsuser->location) . '" '
    . 'name="'      . xe($rsuser->name)     . '" '
    . ' />'
    );
  }
  else
  {
    $this->{web}->set_session_user_from_rs( undef );
    $this->{web}->set_xml_response(
      '<response cmd="'.$cmd.'" result="NOK" reason="invalid user/password" session_id="' . $session->id . '" />'
    );
  }  
}

# only really needed for testing, i.e. test that login check works for other handlers by logging out first
sub cf_logout
{
  my ($this, $cmd, $r) = @_;
  my $user = $session->param('user');
  $user->{is_logged_in}  = 0;
  $session->param('user', $user);
  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="OK" session_id="' . $session->id . '" />'
  );
}

=head2 cf_register - Flash - register

Input: user_email (login), user_password, user_name, user_location

Return XML error if missing parameters.
Return NOK with reason "already registered" if user with login==user_email already exists.
Create user record with supplied data or return NOK with reason "program error: create user".
Send email to user or return NOK with reason "program error: email user".
Return OK.

=cut

sub cf_register
{
  my ($this, $cmd, $r) = @_;
  return if $this->set_xml_error_if_missing_param($cmd, $r, qw/ user_email user_password user_name user_location /);
  my $login = $r->{'user_email'}||'';
  my $pass = $r->{'user_password'}||'';
  my $name = $r->{'user_name'}||'';
  my $location = $r->{'user_location'}||'';
  my $optout = $r->{'user_email_optout'};
  $optout = (lc($optout) eq 'true') ? 'yes' : 'no';

  if ( $c{schema}->resultset('User')->find({ login => $login }) )
  {
    $this->{web}->set_xml_response(
      '<response cmd="'.$cmd.'" result="NOK" reason="already registered" session_id="' . $session->id . '" />'
    );
    return;
  }

  my $rsuser = $c{schema}->resultset('User')->create({
    login     => $login,
    password  => Digest::MD5::md5_hex($pass),
    name      => $name,
    location  => $location,
    email_optout => $optout,
    is_admin  => 'no',
    });

  # login
  $this->{web}->set_session_user_from_rs( $rsuser );

  my $body = <<EOM ;
<html>You have registered on the Sample App website <a href="http://www.sampleapp.com/">www.sampleapp.com</a><br>
<b>Your login:</b> $login<br>
<b>Your password:</b> $pass<br>
<p>
<font size=\"-1\">This is an automatic email from the Sample App website <a href=\"http://www.sampleapp.com/\">www.sampleapp.com</a></font>
</html>
EOM

  my ($code, $msg) = $this->{web}->send_mail(
    to        => $login,
    #to        => $c{'editor_email'},
    #to_name   => $name,
    from      => $c{'reg_mail_from'},
    from_name => $c{'reg_mail_from_name'},
    subject   => 'Sample App Registration',
    body      => $body,
    );  

  unless ($code)
  {
    $this->{web}->set_xml_response(
      '<response cmd="'.$cmd.'" result="NOK" reason="program error: email user" session_id="' . $session->id . '" />'
    );
    print STDERR "app: cf_register: user email: $msg\n"; # TODO log4perl
    return;
  }

  # set logged in here not necessary with Flash frontend
  $this->{web}->set_session_user_from_rs( $rsuser );

  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="OK" session_id="' . $session->id . '" '
    . 'user_email="'. xe($rsuser->login)
    . '" />'
  );    
}

sub cf_reset_password
{
  my ($this, $cmd, $r) = @_;
  return if $this->set_xml_error_if_missing_param($cmd, $r, 'user_email');
  my $login = $r->{user_email}||'';

  if ( my $rsuser = $c{schema}->resultset('User')->find({ login => $login }) )
  {
    my $login = $rsuser->login;
    my $resetlink = $this->resetpasswordurl($login);
    my $body = <<EOM ;
<html>You (or possibly someone pretending to be you) has asked to reset your password on the Sample App website <a href="http://www.sampleapp.com/">www.sampleapp.com</a>
<p>
If this was you, please click the following link <a href="$resetlink">reset my password</a>
or if that doesn't work open a web browser window and type in the following address: $resetlink
<p>
If this was not you, please ignore this email.
<p>
<font size=\"-1\">This is an automatic email from the Sample App website <a href=\"http://www.sampleapp.com/\">www.sampleapp.com</a></font>
</html>
EOM

    my ($code, $msg) = $this->{web}->send_mail(
      to        => $login,
      to_name   => $rsuser->name,
      from      => $c{'reg_mail_from'},
      from_name => $c{'reg_mail_from_name'},
      subject   => 'Sample App Password Reset',
      body      => $body,
      );  
    unless ($code)
    {
      $this->{web}->set_xml_response(
        '<response cmd="'.$cmd.'" result="NOK" reason="program error: email user" session_id="' . $session->id . '" />'
      );
      print STDERR "app: cf_reset_password: user email: $msg\n"; # TODO log4perl
      return;
    }
  }
  # if no such user, still send OK result to fox spammers

  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="OK" session_id="' . $session->id . '" />'
  );    
}

sub cf_faqs_list
{
  my ($this, $cmd, $r) = @_;

  my $faqs = '';
  # faq.id question answer create_dt
  for ( $c{schema}->resultset('Faq')->search({}, { order_by => 'id DESC' })->all )
  {
    $faqs .= '<faq id="'.xe($_->get_column('id'))
    . '" faq_question="'.xe($_->get_column('question'))
    . '" faq_answer="'.xe($_->get_column('answer'))
    . '" faq_location="'.xe($_->get_column('location'))
    . '" faq_name="'.xe($_->get_column('name'))
    . '" />';
  }  

  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="OK" session_id="'. $session->id .'" />'
  . '<faqslist>'
  . $faqs
  . '</faqslist>'
  );
}

sub cf_faqs_ask
{
  my ($this, $cmd, $r) = @_;
  return if $this->set_xml_error_if_not_logged_in($cmd);
  return if $this->set_xml_error_if_missing_param($cmd, $r, qw/faq_question faq_name faq_location/ );
  my $user_email = $session->param('user')->{login};
  my $question  = $r->{'faq_question'}||'';
  my $name = $r->{'faq_name'}||'';
  my $location = $r->{'faq_location'}||'';

  # stash asked faq in case email goes astray
  $c{schema}->resultset('FaqAsked')->create({
    question  => $question,
    login     => $user_email,
    name      => $name,
    location  => $location,
    });

  # email to web editor
  my $curtime = curtime();
  my $body = <<EOM ;
<html>The following question was asked on the FAQ page<br>
<br>
<b>Date:</b> $curtime<br>
<b>User:</b> $user_email<br>
<b>Name:</b> $name<br>
<b>Location:</b> $location</br>
<b>Question:</b> $question
<p>
<font size=\"-1\">This is an automatic email from the Sample App website <a href=\"http://www.sampleapp.com/app/\">www.sampleapp.com</a></font>
</html>
EOM

  my ($code, $msg) = $this->{web}->send_mail(
    to        => $c{'editor_email'},
    #to_name   => ,
    from      => $c{'faq_mail_from'},
    from_name => $c{'faq_mail_from_name'},
    subject   => 'Sample App Faq',
    body      => $body,
    );  
  unless ($code)
  {
    $this->{web}->set_xml_response(
      '<response cmd="'.$cmd.'" result="NOK" reason="program error: email failed" session_id="' . $session->id . '" />'
    );
    print STDERR "app: cf_faqs_ask: email: $msg\n"; # TODO log4perl
    return;
  }

  $this->{web}->set_xml_response(
    '<response cmd="'.$cmd.'" result="OK" session_id="'. $session->id . '" />'
  );
}

1;
__END__

=head1 NAME

App::Controller

=head1 DESCRIPTION

Controllers for Sample App website

=head1 AUTHOR

Peter Edwards of Dragonstaff Limited <peter@dragonstaff.co.uk>

=head1 COPYRIGHT

Copyright (c) 2007 Dragonstaff Limited.

All rights reserved.

=cut
