#!/usr/bin/perl -w
# App/Widget.pm

package App::Widget;

use strict;

=head1 Widget - Form widgets for Template Toolkit

=cut

use Carp;
use Data::Dumper;

use base qw( Template::Plugin );

use vars qw(%c $session $cgi);
*c = \*app::c;
*session = \$app::session;
*cgi = \$app::cgi;

sub new
{
   my $class = shift;
   my $context = shift;
   my $this = {
   	_CONTEXT => $context,
		@_,
		};
   bless($this, $class);
}

=head1
sub load
{
	my ($class, $context) = @_;
	$class;
}

sub error
{
	my $this = shift;
	print "error\n";
	confess "TTB:@_";
}
=cut

sub load_datadict
{
  my $this = shift;
  return if $c{datadict};
  # 1. screen, table, field
  # 2. table, field
  # 3. screen, field
  # 4. field
  my $dict = { screen_table => {}, screen => {}, table => {}, field => {} };
  for ( $c{schema}->resultset('Datadict')->search({})->all )
  {
    my $entry = {
      field_type => $_->field_type,
      label => $_->label,
      url => $_->url,
      input => $_->input,
      select_choices => $_->select_choices,
      };
=head1
    if ( $_->screen && $_->table )
    {
      $dict->{screen_table}->{$_->screen}->{$_->table}->{$_->field} = $entry;
    }
    elsif ( $_->table )
    { 
      $dict->{table}->{$_->table}->{$_->field} = $entry;
    }
    elsif ( $_->screen )
    {
      $dict->{screen}->{$_->screen}->{$_->field} = $entry;
    }
    else
=cut
    if ( $_->table )
    { 
      $dict->{table}->{$_->table}->{$_->field} = $entry;
    }
    else
    {
      $dict->{field}->{$_->field} = $entry;
    }  
  }  
  $c{datadict} = $dict;
}

sub dict_get_label
{
	my $this = shift;
	my $p = shift;
	my $field = $p->{field}||'Unknown';
	my $table = $p->{table}||'';
	$c{datadict} || $this->load_datadict;
	# TODO hierarchy of precedence from most to least specific dictonary entry
  # 1. screen, table, field
  # 2. table, field
  # 3. screen, field
  # 4. field
  # For now just held at field level
  return
      $c{datadict}->{table}->{$table}->{$field}->{label}
  ||  $c{datadict}->{field}->{$field}->{label}
  ||  "Lbl $field";
}

sub dict_format_value
{
  my $this = shift;
  my $p = shift;
  '';
}

sub field
{
  my $this = shift;
  my $p = shift;
  my $field = $p->{field}||confess;
  my $value = $p->{value}||'';
  my $table = $p->{table}||'';
  my $screen = $p->{screen}||'';
	my $d = $c{datadict}->{table}->{$table}->{$p->{field}} || $c{datadict}->{field}->{$p->{field}} || {};
	if ( $d->{url} )
	{
	  if ( $d->{url} =~ m/^\// )
	  {
	    '<a href='
	    . sprintf("$d->{url}", $value, $value, $value, $value, $value)
	    . '>' .$value . '</a>';
	  }
	  elsif ( $d->{url} =~ m/^http:/)
	  {
	    '<a href='
	    . sprintf("$d->{url}", $value, $value, $value, $value, $value)
	    . '>' .$value . '</a>';
	  }
	  else
	  { 
	    '<a href='
	    . $c{baseurl}
	    . sprintf("$d->{url}", $value, $value, $value, $value, $value)
	    . '>' .$value . '</a>';
	  }
  }	
  else
  {
    "$value";
  }
}

sub input
{
  my $this = shift;
  my $p = shift;
  my $field = $p->{field}||confess;
  my $dictfield = $p->{dictfield} || $field;
  my $value = $p->{value}||'';
  my $class = $p->{class}||'';
  my $table = $p->{table}||'';
  my $screen = $p->{screen}||'';
  my $noblank = $p->{noblank} ? 1 : 0;
	my $d = $c{datadict}->{table}->{$table}->{$dictfield} || $c{datadict}->{field}->{$dictfield} || {};
	$class = " class=\"$class\"" if $class;
	# TODO use datadict input and select_choices
	if ( ($d->{input}||'') eq 'select' )
	{
	  $this->input_select({ field => $field, dictfield => $dictfield, value => $value, class => $class, noblank => $noblank, options => $d->{select_choices} });
  }	
  else
  {
	  "<input type=\"text\" name=\"$field\" value=\"$value\" $class />";
	}
}

sub input_select
{
  my $this = shift;
  my $p = shift;
  my $field = $p->{field}||confess;
  my $value = $p->{value}||'';
  my $class = $p->{class}||'finput';
  my $options = $p->{options}||'';
  my $noblank = $p->{noblank} ? 1 : 0;
  my $opts = '';
  $options = ';'.$options unless $noblank;
  for ( split /\;/, $options )
  {
    my ($code, $label) = split /\=/, $_;
    unless ( defined $code )
    {
      $code = '';
      $label = '--select--';
    }
    $label = $code unless defined $label;
    $opts .= '<option value="'.$code.'"';
    $opts .= ' selected' if $code eq $value;
    $opts .= '>'.$label.'</option>';
  }
  '<select class="'.$class.'" name="'.$field.'">' .$opts. '</select>';
}

sub hidden_field
{
	my $this = shift;
	my $p = shift;
	my $name = $p->{name};
	my $value = $p->{value} || '';
	"<input type=\"hidden\" id=\"$name\" name=\"$name\" value=\"$value\" />";
}

sub form
{
  my $this = shift;
  my $p = shift;
  
}

sub set_context
{
}

sub button
{
  my $this = shift;
  my $p = shift;
	my $name = $p->{name};
	my $value = $p->{value} || '';
	my $label = $p->{label} || $name;
	my $s = <<EOM ;
<button name="$name" id="$name" type="button" value="$value" onclick="return btnclick($name,$value);">$label</button>
EOM
#  unless ( $c{widget}->{button_init} )
#  {
#    $c{widget}->{button_init} = 1;
#    $s .= $this->hidden_field(name => 'button_name')
#    .     $this->hidden_field(name => 'button_value');
#  }
  $s;
}

sub button_view
{
	confess;
}

sub record_selectors
{
	confess;
}

sub escape_javascript
{
  my $this = shift;
  my $p = shift;
  #my @a = $p ? (%$p) : ();
  my $s = shift @$p || return '';
  my %map = (
    "\\"      =>   "\\\\",
    "\'"      =>   "\\\'",
    "\""      =>   "\\\"",
    "\@"      =>   "\\0100",
  );
  $s =~ s{ ( [\Q\\'"@\E] ) } { $map{$1} }gsex;
  $s =~ s/((\r\n)|(\n))/\\r/g;
  $s;
}

sub tt_escape_html
{
  my $this = shift;
  my $p = shift;
  my $s = shift @$p || return '';
  my %map = (
    "\<"      =>   "&lt;",
    "\>"      =>   "&gt;",
    "\&"      => "&amp;",
    "\""      =>   "&quot;",
    "\@"      => "&#064;",
    "\'"      => "&#146;",
    );
  no warnings;
  $s =~ s{ ( [\Q<>&"'@\E] ) } { $map{$1} if $1 }gsex;
  use warnings;
  $s;
}

sub debug
{
	return unless $_[1];
	shift;
	print STDERR Dumper(@_);
	return;
}

1;

__END__
