#!/usr/bin/perl
# test/t_web.pl

=head1 t_web.pl - test Web XHTML handler

=cut

use warnings;
use strict;

use Test::More qw(no_plan);

use XML::Simple qw/:strict/;
use LWP::UserAgent;
use HTTP::Request::Common;
use HTTP::Cookies;
use DBI;
use Carp;

my $server = 'http://centos41';
my $user_login = 'user@somedomain.com';
my $user_password = 'user';
my ($message, $response, $expect);
my $dbh;
my $ua;
my $cookie_jar;

{
  $cookie_jar = HTTP::Cookies->new( file => './cookies.dat', ignore_discard => 1, autosave => 1 );
  $ua = LWP::UserAgent->new( agent => 'perl post' );
  $ua->cookie_jar($cookie_jar);

	$dbh = DBI->connect("DBI:mysql:app", "app", "app") ||confess "cannot connect to mysql database";

  #login();
  t_login();

  $dbh->disconnect;
  $dbh = undef;
}

#===

sub postit
{ 
  my $message = shift;
  my $not_logged_in = shift;
  if ( $not_logged_in )
  {
    $cookie_jar->save;
    $cookie_jar->clear;
  }  
  #"<?xml version=\"1.0\" encoding=\"UTF-8\"?>
  my $response = $ua->request(POST
    $server.'/app/',
    #Content_Type => 'text/xml',
    Content => $message
    );
  if ( $not_logged_in )
  {
    $cookie_jar->revert;
  }
  else
  {
    $cookie_jar->extract_cookies($response);
    $cookie_jar->save;
  }
  $response;
}

sub do_sql
{
	my $s = shift || confess;
	$s =~ s/[\000-\037]/ /g; # nonprint to space
	$s =~ s/;[\s]*$//; # no end ;
	my $sth = $dbh->prepare( $s ) || confess "cannot prepare sql: $dbh->errstr";
	$sth->execute(@_) || confess "cannot execute sql: " . $dbh->errstr;
	#print $sth->rows, " rows affected\n";
}

sub login
{
  $message = '<request cmd="login" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{<response cmd="login" result="OK"};
  $response = postit($message);
  confess "cannot login" unless $response->as_string =~ m/$expect/;
  diag 'logged in';
}

sub logout
{
  $message = '<request cmd="logout" />';
  $expect = qq{<response cmd="logout" result="OK"};
  $response = postit($message);
  confess "cannot logout" unless $response->as_string =~ m/$expect/;
  diag 'logged out';
}

#===

sub t_login
{
  $message = '<request cmd="login_wrong_command" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{Flash handler unknown cmd};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'invalid command gives Flash handler error' ) || print $response->as_string;

  $message = '<request cmd="login" user_email="fred@bloggs.com" user_password="bloggs" />';
  $expect = qq{<response cmd="login" result="NOK" reason="invalid user/password"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'login - invalid username/password gives NOK' ) || print $response->as_string;

  $message = '<request cmd="login" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{<response cmd="login" result="OK"};
  $response = postit($message);
  ok ( $response->is_success, 'Flash XML handler responds to known command login' );
  ok ( ($response->as_string =~ m/$expect/), 'login - valid username/password gives OK' ) || print $response->as_string;
}

__END__
